"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _stringify = _interopRequireDefault(require("@babel/runtime/core-js/json/stringify"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _weakSet = _interopRequireDefault(require("@babel/runtime/core-js/weak-set"));

var _map = _interopRequireDefault(require("@babel/runtime/core-js/map"));

var _includes = _interopRequireDefault(require("@babel/runtime/core-js/array/includes"));

var _symbol = _interopRequireDefault(require("@babel/runtime/core-js/symbol"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _generator = _interopRequireDefault(require("@babel/generator"));

var _conf = require("@sector-labs/lingui-conf");

// Map of messages
var MESSAGES = (0, _symbol.default)("I18nMessages"); // We need to remember all processed nodes. When JSX expressions are
// replaced with CallExpressions, all children are traversed for each CallExpression.
// Then, i18n._ methods are visited multiple times for each parent CallExpression.

var VISITED = (0, _symbol.default)("I18nVisited");

function addMessage(path, messages, _ref) {
  var id = _ref.id,
      defaults = _ref.defaults,
      origin = _ref.origin;

  if (messages.has(id)) {
    var message = messages.get(id); // only set/check default language when it's defined.

    if (message.defaults && defaults && message.defaults !== defaults) {
      throw path.buildCodeFrameError("Different defaults for the same message ID.");
    } else {
      if (defaults) {
        message.defaults = defaults;
      }

      ;
      [].push.apply(message.origin, origin);
    }
  } else {
    messages.set(id, {
      defaults: defaults,
      origin: origin
    });
  }
}

function _default(_ref2) {
  var t = _ref2.types;
  var localTransComponentName;
  var opts = (0, _conf.getConfig)();
  var optsBaseDir = opts.rootDir;

  function isTransComponent(node) {
    return t.isJSXElement(node) && t.isJSXIdentifier(node.openingElement.name, {
      name: localTransComponentName
    });
  }

  var isNoopMethod = function isNoopMethod(node) {
    return t.isIdentifier(node, {
      name: "i18nMark"
    });
  };

  var isI18nMethod = function isI18nMethod(node) {
    return t.isMemberExpression(node) && t.isIdentifier(node.object, {
      name: "i18n"
    }) && t.isIdentifier(node.property, {
      name: "_"
    });
  };

  function collectMessage(path, file, props) {
    var messages = file.get(MESSAGES);

    var filename = _path.default.relative(optsBaseDir, file.opts.filename).replace(/\\/g, "/");

    var line = path.node.loc ? path.node.loc.start.line : null;
    props.origin = [[filename, line]];
    addMessage(path, messages, props);
  }

  return {
    visitor: {
      // Get the local name of Trans component. Usually it's just `Trans`, but
      // it might be different when the import is aliased:
      // import { Trans as T } from '@sector-labs/lingui-react';
      ImportDeclaration: function ImportDeclaration(path) {
        var node = path.node;
        var moduleName = node.source.value;
        if (!(0, _includes.default)(["@sector-labs/lingui-react", "@lingui/core"], moduleName)) return;
        var importDeclarations = {};

        if (moduleName === "@sector-labs/lingui-react") {
          node.specifiers.forEach(function (specifier) {
            importDeclarations[specifier.imported.name] = specifier.local.name;
          });

          if (importDeclarations["Trans"]) {
            localTransComponentName = importDeclarations["Trans"];
          }
        } // Remove imports of i18nMark identity


        node.specifiers = node.specifiers.filter(function (specifier) {
          return specifier.imported.name !== "i18nMark";
        });

        if (!node.specifiers.length) {
          path.remove();
        }
      },
      // Extract translation from <Trans /> component.
      JSXElement: function JSXElement(path, _ref3) {
        var file = _ref3.file;
        var node = path.node;
        if (!localTransComponentName || !isTransComponent(node)) return;
        var attrs = node.openingElement.attributes || [];
        var props = attrs.reduce(function (acc, item) {
          var key = item.name.name;
          if (key === "id" || key === "defaults") acc[key] = item.value.value;
          return acc;
        }, {});

        if (!props.id) {
          // <Trans id={message} /> is valid, don't raise warning
          var idProp = attrs.filter(function (item) {
            return item.name.name === "id";
          })[0];

          if (idProp === undefined || t.isLiteral(props.id)) {
            console.warn("Missing message ID, skipping.");
            console.warn((0, _generator.default)(node).code);
          }

          return;
        }

        collectMessage(path, file, props);
      },
      // Extract translation from i18n._ call
      CallExpression: function CallExpression(path, _ref4) {
        var file = _ref4.file;
        var node = path.node;
        var visited = file.get(VISITED);

        if ( // we already visited this node
        visited.has(node.callee) || // nothing to extract
        !isI18nMethod(node.callee) && !isNoopMethod(node.callee)) {
          return;
        }

        visited.add(node.callee);

        if (isNoopMethod(node.callee) && !t.isStringLiteral(node.arguments[0])) {
          console.warn("Only string literals are allowed in i18nMark.");
          return;
        }

        var attrs = node.arguments[1] && node.arguments[1].properties ? node.arguments[1].properties : [];
        var idArg = node.arguments[0];
        var id = idArg && idArg.value;

        if (!id) {
          // i18n._(message) is valid, don't raise warning
          if (idArg === undefined || t.isLiteral(idArg)) {
            console.warn("Missing message ID, skipping.");
            console.warn((0, _generator.default)(node).code);
          }

          return;
        }

        var props = attrs.reduce(function (acc, item) {
          var key = item.key.name;
          if (key === "defaults") acc[key] = item.value.value;
          return acc;
        }, {
          id: id
        });
        collectMessage(path, file, props);

        if (isNoopMethod(node.callee)) {
          var translation = node.arguments[0];
          path.replaceWith(t.stringLiteral(translation.value));
        }
      }
    },
    pre: function pre(file) {
      localTransComponentName = null; // Ignore else path for now. Collision is possible if other plugin is
      // using the same Symbol('I18nMessages').
      // istanbul ignore else

      if (!file.has(MESSAGES)) {
        file.set(MESSAGES, new _map.default());
      }

      file.set(VISITED, new _weakSet.default());
    },
    post: function post(file) {
      /* Write catalog to directory `localeDir`/_build/`path.to.file`/`filename`.json
       * e.g: if file is src/components/App.js (relative to package.json), then
       * catalog will be in locale/_build/src/components/App.json
       */
      var localeDir = this.opts.localeDir || opts.localeDir;
      var filename = file.opts.filename;

      var baseDir = _path.default.dirname(_path.default.relative(optsBaseDir, filename));
      /* if the path is relative and reverse to a directory outside
       * the root, then we have to escape the ".." so that we still
       * write to the build dir */


      var escapedBaseDir = baseDir.replace(/\./g, "\\.");

      var targetDir = _path.default.join(localeDir, "_build", escapedBaseDir);

      var messages = file.get(MESSAGES);
      var catalog = {}; // no messages, skip file

      if (!messages.size) return;
      messages.forEach(function (value, key) {
        catalog[key] = value;
      });

      _mkdirp.default.sync(targetDir);

      var _fsPath$basename$spli = _path.default.basename(filename).split(".", 2),
          _fsPath$basename$spli2 = (0, _slicedToArray2.default)(_fsPath$basename$spli, 1),
          basename = _fsPath$basename$spli2[0];

      _fs.default.writeFileSync(_path.default.join(targetDir, "".concat(basename, ".json")), (0, _stringify.default)(catalog, null, 2));
    }
  };
}
